@@ -1,6 +1,5 @@
 package urpm::download;
 
-# $Id: download.pm 271299 2010-11-21 15:54:30Z peroyvind $
 
 use strict;
 use urpm::msg;
@@ -10,6 +9,9 @@ use Cwd;
 use Exporter;
 # perl_checker: require urpm
 
+# help perl_checker:
+sub getcwd { goto &Cwd::getcwd }
+
 our @ISA = 'Exporter';
 our @EXPORT = qw(get_proxy
 	propagate_sync_callback
@@ -278,7 +280,7 @@ sub set_proxy {
 	if (my ($http_proxy) = $p->{http_proxy} && parse_http_proxy($p->{http_proxy})) {
 	    my $allproxy = $p->{user}; 
 	    $allproxy .= ":" . $p->{pwd} if $p->{pwd}; 
-	    $allproxy .= "@";
+	    $allproxy .= "@" if $p->{user};
 	    $allproxy .= $http_proxy;
 	    @res = ("--all-proxy=http://$allproxy");
 	}
@@ -538,7 +540,7 @@ sub _curl_action {
 			}
 			#- this checks that download has actually started
 			if ($_ eq "\n"
-			    && !($speed eq 0 && $percent == 100 && index($eta, '--') >= 0) #- work around bug 13685
+			    && !($speed == 0 && $percent == 100 && index($eta, '--') >= 0) #- work around bug 13685
 			) {
 			    propagate_sync_callback($options, 'end', $file);
 			    $file = undef;
@@ -586,7 +588,7 @@ sub sync_rsync {
 	    my $buf = '';
 	    my $cmd = join(" ", "/usr/bin/rsync",
 		($limit_rate ? "--bwlimit=$limit_rate" : @{[]}),
-		($options->{quiet} ? qw(-q) : qw(--progress -v)),
+		($options->{quiet} ? qw(-q) : qw(--progress -v --no-human-readable)),
 		($options->{compress} ? qw(-z) : @{[]}),
 		($options->{ssh} ? qq(-e $options->{ssh}) : 
 		   ("--timeout=$CONNECT_TIMEOUT",
@@ -694,7 +696,7 @@ sub sync_aria2 {
 	"/usr/bin/aria2c", $options->{debug} ? ('--log', "$options->{dir}/.aria2.log") : @{[]},
 	'--auto-file-renaming=false',
 	'--ftp-pasv',
-	'--summary-interval=0',
+	'--summary-interval=1',
 	'--follow-metalink=mem',
       $medium->{mirrorlist} ? (
 	'--metalink-enable-unique-protocol=true', # do not try to connect to the same server using the same protocol
@@ -762,8 +764,29 @@ sub _parse_aria2_output {
 				propagate_sync_callback($options, 'start', $file)
 				  if !$options->{is_retry};
 			}
-			#parses aria2c: [#1 SIZE:176.0KiB/2.5MiB(6%) CN:3 SPD:256.22KiBs ETA:09s]
-    		    if ($buf =~ m!^\[#\d*\s+\S+:([\d\.]+\w*).([\d\.]+\w*)\S([\d]+)\S+\s+\S+\s*([\d\.]+)\s\w*:([\d\.]+\w*)\s\w*:(\d+\w*)\]$!) {
+
+			# aria2c 1.16 and beyond:
+			# parses aria2c: [#2c8dae 496KiB/830KiB(59%) CN:1 DL:84KiB ETA:3s]
+			#
+			# using multiline mode and comments for better readability:
+			#
+			if ($buf =~ m!
+				^\[\#[\dA-Fa-f]+ # match #2c8dae
+				\s+
+				([\d\.]+\w*) # Match 496KiB
+					/
+				([\d\.]+\w*) # Match 830KiB
+				\s* \( (\d+) % \) # Match (59%)
+				\s+
+				CN:(\S+) # Match CN:1
+				\s+
+				DL:(\S+) # Match DL:84KiB
+				\s+
+				ETA:(\w+)
+				\]$
+				!msx
+			)
+			{
 			    my ($total, $percent, $speed, $eta) = ($2, $3, $5, $6);
 			    #- $1 = current downloaded size, $4 = connections
 		    if (propagate_sync_callback($options, 'progress', $file, $percent, $total, $eta, $speed) eq 'canceled') {
@@ -816,11 +839,13 @@ END {
 
 #- get the width of the terminal
 my $wchar = 79;
-eval {
+if (-t *STDOUT) {
+  eval {
     require Term::ReadKey;
     ($wchar) = Term::ReadKey::GetTerminalSize();
     --$wchar;
-};
+  };
+}
 
 sub progress_text {
     my ($mode, $percent, $total, $eta, $speed) = @_;
@@ -918,14 +943,22 @@ sub sync_rel {
     my @result_files = map { $all_options->{dir} . '/' . basename($_) } @$rel_files;
     unlink @result_files if $all_options->{preclean};
 
+    #- The files must be world-readable, else mgaapplet and urpm* commands run as
+    #- a normal user won't be able to read them. We enforce umask here in the case
+    #- where the msec security level is set to 'secure' (which means umask 077)
+    #- or where we are run from a gdm-x-session (mga#24636)
+    my $old_umask = umask 0022;
+
     (my $cwd) = getcwd() =~ /(.*)/;
     eval { _sync_webfetch_raw($urpm, $medium, $rel_files, \@files, $all_options) };
     my $err = $@;
     chdir $cwd;
+    umask $old_umask;
     if (!$err) {
 	$urpm->{log}(N("retrieved %s", $files_text));
 	\@result_files;
     } else {
+	$urpm->{log}("error: $err");
 	# don't leave partial download
 	unlink @result_files;
 	undef;
@@ -1087,7 +1120,6 @@ sub _create_metalink_ {
 
 1;
 
-__END__
 
 =back
 
@@ -1097,4 +1129,6 @@ Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005 MandrakeSoft SA
 
 Copyright (C) 2005-2010 Mandriva SA
 
+Copyright (C) 2011-2017 Mageia
+
 =cut
